@@ -16,7 +16,7 @@ lsode <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
   maxord=NULL, bandup=NULL, banddown=NULL, maxsteps=5000, 
   dllname=NULL,initfunc=dllname, initpar=parms, 
   rpar=NULL, ipar=NULL, nout=0, outnames=NULL,forcings=NULL,
-  initforc = NULL, fcontrol=NULL, ...)
+  initforc = NULL, fcontrol=NULL, events=NULL, ...)
 {
 
 ### check input
@@ -62,6 +62,8 @@ lsode <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
   Ynames    <- attr(y,"names")
   flist     <- list(fmat=0,tmat=0,imat=0,ModelForc=NULL)
   ModelInit <- NULL
+  Eventfunc <- NULL
+  events <- checkevents(events, times, Ynames, dllname) 
 
   if (is.character(func)) {   # function specified in a DLL
     DLL <- checkDLL(func,jacfunc,dllname,
@@ -79,7 +81,7 @@ lsode <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
     rho <- NULL
     if (is.null(ipar)) ipar<-0
     if (is.null(rpar)) rpar<-0
-
+    Eventfunc <- events$func
   } else {
 
     if (is.null(initfunc))
@@ -104,6 +106,12 @@ lsode <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
         attr(state,"names") <- Ynames
         jacfunc(time,state,parms,...)
       }
+      if (! is.null(events$Type))
+      if (events$Type == 2)
+         Eventfunc <- function(time,state) {
+           attr(state,"names") <- Ynames
+           events$func(time,state,parms,...) 
+         }
     } else {                          # no ynames...
       Func    <- function(time,state)
         func   (time,state,parms,...)[1]
@@ -113,6 +121,12 @@ lsode <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
          
       JacFunc <- function(time,state)
         jacfunc(time,state,parms,...)
+
+      if (! is.null(events$Type))
+       if (events$Type == 2)
+         Eventfunc <- function(time,state)  
+           events$func(time,state,parms,...) 
+
     }
         
     ## Check function and return the number of output variables +name
@@ -120,6 +134,10 @@ lsode <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
     Nglobal<-FF$Nglobal
     Nmtot <- FF$Nmtot
 
+    if (! is.null(events$Type))
+      if (events$Type == 2) 
+        checkEventFunc(Eventfunc,times,y,rho)
+
     if (miter %in% c(1,4)) {
       tmp <- eval(JacFunc(times[1], y), rho)
       if (!is.matrix(tmp))
@@ -206,12 +224,12 @@ lsode <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
   IN <-2
     
   out <- .Call("call_lsoda",y,times,Func,initpar,
-               rtol, atol, rho, tcrit, JacFunc, ModelInit,  
+               rtol, atol, rho, tcrit, JacFunc, ModelInit, Eventfunc,  
                as.integer(verbose), as.integer(itask), as.double(rwork),
                as.integer(iwork), as.integer(imp),as.integer(Nglobal),
                as.integer(lrw),as.integer(liw),as.integer(IN),
                NULL, as.integer(0), as.double (rpar), as.integer(ipar),
-               as.integer(0), flist, PACKAGE="deSolve")
+               as.integer(0), flist, events, PACKAGE="deSolve")
 
 ### saving results
 
