@@ -13,9 +13,8 @@ lsodes <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
   verbose=FALSE, tcrit = NULL, hmin=0, hmax=NULL, hini=0, ynames=TRUE,
   maxord=NULL, maxsteps=5000, lrw=NULL, liw=NULL, 
   dllname=NULL, initfunc=dllname,
-  initpar=parms, rpar=NULL, ipar=NULL, 
-  nout=0, outnames=NULL,forcings=NULL,
-  initforc = NULL, fcontrol=NULL, ...)  {
+  initpar=parms, rpar=NULL, ipar=NULL, nout=0, outnames=NULL, forcings=NULL,
+  initforc = NULL, fcontrol=NULL, events=NULL, ...)  {
 
 ### check input
   hmax <- checkInput (y, times, func, rtol, atol,
@@ -107,6 +106,9 @@ lsodes <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
   Ynames    <- attr(y,"names")
   flist     <- list(fmat=0,tmat=0,imat=0,ModelForc=NULL)
   ModelInit <- NULL
+  Eventfunc <- NULL
+  
+  events <- checkevents(events, times, Ynames, dllname) 
 
   if (is.character(func)) {   # function specified in a DLL
     DLL <- checkDLL(func,jacvec,dllname,
@@ -124,7 +126,7 @@ lsodes <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
     rho <- NULL
     if (is.null(ipar)) ipar<-0
     if (is.null(rpar)) rpar<-0
-
+    Eventfunc <- events$func
   } else {
 
     if(is.null(initfunc))
@@ -148,6 +150,12 @@ lsodes <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
         attr(state,"names") <- Ynames
         jacvec(time,state,J,parms,...)
       }
+       if (! is.null(events$Type))
+       if (events$Type == 2)
+         Eventfunc <- function(time,state) {
+           attr(state,"names") <- Ynames
+           events$func(time,state,parms,...) 
+         }
     } else {                            # no ynames...
       Func    <- function(time,state)
         func   (time,state,parms,...)[1]
@@ -157,12 +165,22 @@ lsodes <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
          
       JacFunc <- function(time,state,J)
         jacvec(time,state,J,parms,...)
+     
+      if (! is.null(events$Type))
+      if (events$Type == 2)
+         Eventfunc <- function(time,state)  
+           events$func(time,state,parms,...) 
     }
         
     ## Check function and return the number of output variables +name
     FF <- checkFunc(Func2,times,y,rho)
     Nglobal<-FF$Nglobal
     Nmtot <- FF$Nmtot
+
+    if (! is.null(events$Type))
+      if (events$Type == 2) 
+        checkEventFunc(Eventfunc,times,y,rho)
+    
   }
 
 ### work arrays iwork, rwork
@@ -277,12 +295,12 @@ lsodes <- function(y, times, func, parms, rtol=1e-6, atol=1e-6,
   IN <-3
 
   out <- .Call("call_lsoda",y,times,Func,initpar,
-               rtol, atol, rho, tcrit, JacFunc, ModelInit,
+               rtol, atol, rho, tcrit, JacFunc, ModelInit, Eventfunc,
                as.integer(verbose), as.integer(itask), as.double(rwork),
                as.integer(iwork), as.integer(imp),as.integer(Nglobal),
                as.integer(lrw),as.integer(liw),as.integer(IN),
                NULL, as.integer(0), as.double (rpar), as.integer(ipar),
-               as.integer(Type),flist, PACKAGE="deSolve")
+               as.integer(Type),flist, events, PACKAGE="deSolve")
 
 ### saving results    
   out <- saveOut(out, y, n, Nglobal, Nmtot, func, Func2,
