@@ -158,6 +158,31 @@ rkMethod <- function(method = NULL, ...) {
                 -1453857185.0/822651844.0, 69997945.0/29380423.0),
          stage = 7,
          Qerr  = 4
+    ),
+    ## Runge-Kutta-Fehlberg 78 method
+    rk78f = list(ID = "rk78f",
+        varstep = TRUE,
+        FSAL    = FALSE,
+        A  = matrix(
+         c(rep(0,12),
+         2/27, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+         1/36, 1/12, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+         1/24, 0, 1/8, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+         5/12, 0, -25/16, 25/16, 0, 0, 0, 0, 0, 0, 0, 0,
+         0.05, 0, 0, 0.25, 0.2, 0, 0, 0, 0, 0, 0, 0,
+         -25/108, 0, 0, 125/108, -65/27, 125/54, 0, 0, 0, 0, 0, 0,
+         31/300, 0, 0, 0, 61/225, -2/9, 13/900, 0, 0, 0, 0, 0,
+         2, 0, 0, -53/6, 704/45, -107/9, 67/90, 3, 0, 0, 0, 0,
+         -91/108, 0, 0, 23/108, -976/135, 311/54, -19/60, 17/6, -1/12, 0, 0, 0,
+         2383/4100, 0, 0, -341/164, 4496/1025, -301/82, 2133/4100, 45/82, 45/164, 18/41, 0, 0,
+         3/205, 0, 0, 0, 0, -6/41, -3/205, -3/41, 3/41, 6/41, 0, 0,
+         -1777/4100, 0, 0, -341/164, 4496/1025, -289/82, 2193/4100, 51/82, 33/164, 12/41, 0, 1
+        ), nrow=13, ncol=12, byrow = TRUE),
+        b1 = c(41/840, 0,0,0,0, 34/105, 9/35, 9/35, 9/280, 9/280, 41/840, 0, 0),
+        b2 = c(0, 0, 0, 0, 0, 34/105, 9/35, 9/35, 9/280, 9/280, 0, 41/840, 41/840),
+        c  = c(0, 2./27., 1/9, 1/6, 5/12, 0.5, 5/6, 1/6, 2/3, 1/3, 1, 0, 1),
+        stage = 7,
+        Qerr  = 8
     )
   )
   ## look if the method is known; ode23 and ode45 are used as synonyms
